@@ -18,6 +18,7 @@ __docformat__ = "restructuredtext"
 # import standard packages
 
 # import Zope3 interfaces
+from z3c.json.interfaces import IJSONWriter
 from z3c.language.switch.interfaces import II18n
 
 # import local interfaces
@@ -30,9 +31,10 @@ from ztfy.skin.interfaces import IDefaultView, IContainedDefaultView
 from ztfy.skin.layer import IZTFYBackLayer
 
 # import Zope3 packages
-from z3c.form import field
+from z3c.form import field, button
+from z3c.formjs import jsaction, ajax
 from z3c.table.column import Column, FormatterColumn, GetAttrColumn
-from zope.component import adapts, queryMultiAdapter
+from zope.component import adapts, queryMultiAdapter, getUtility
 from zope.i18n import translate
 from zope.interface import implements, Interface
 from zope.traversing import namespace
@@ -119,6 +121,42 @@ class SchedulerTaskEditForm(EditForm):
         return result
 
 
+class SchedulerTaskExecuteFormMenu(JsMenuItem):
+
+    title = _(" :: Execute now...")
+
+
+class ISchedulerTaskExecuteFormButtons(Interface):
+
+    run = jsaction.JSButton(title=_("Run now !"))
+    cancel = jsaction.JSButton(title=_("Cancel"))
+
+
+class SchedulerTaskExecuteForm(DialogEditForm):
+
+    legend = _("Task immediate run")
+    help = _("You can run this task immediately. Execution log will be available in task's history, if enabled.\n"
+             "WARNING: Task will be run even if it's disabled in it's scheduling settings!")
+
+    fields = field.Fields(Interface)
+    buttons = button.Buttons(ISchedulerTaskExecuteFormButtons)
+
+    @jsaction.handler(buttons['run'])
+    def run_handler(self, event, selector):
+        return '$.ZTFY.form.edit(this.form);'
+
+    @jsaction.handler(buttons['cancel'])
+    def cancel_handler(self, event, selector):
+        return '$.ZTFY.dialog.close();'
+
+    @ajax.handler
+    def ajaxEdit(self):
+        self.context.launch()
+        writer = getUtility(IJSONWriter)
+        return writer.write({ 'output': u'OK',
+                              'message': translate(_("Task successfully scheduled. Execution will start in 5 seconds."), context=self.request) })
+
+
 class TaskScheduleFormMenu(JsMenuItem):
 
     title = _(" :: Schedule...")
